@@ -5,6 +5,8 @@ Lisz Demo App main app base (ae.gui_app) mixin class
 This module is used for to provide common data structures, functions and methods for the various
 demo applications based on the gui_app and the related GUI framework implementation portions
 (like e.g. ae.kivy_app and ae.enaml_app) of the ae namespace.
+
+TODO: add unit tests for 100 % coverage.
 """
 from abc import abstractmethod, ABC
 from typing import Any, Dict, List, Optional, Sequence
@@ -13,11 +15,11 @@ from ae.core import DEBUG_LEVEL_VERBOSE
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action       # type: ignore
 
 
-__version__ = '0.0.4'
+__version__ = '0.0.5'
 
 
-FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper
-FLOW_PATH_TEXT_SEP = " / "           #: flow path separator used by :meth:`~LiszDataMixin.flow_path_text`
+FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper and for drop onto leave item button
+FLOW_PATH_TEXT_SEP = " / "           #: flow path separator for :meth:`~LiszDataMixin.flow_path_text` and drop enter
 
 HELP_TEXT = (
     "Touch item name for to toggle selection.\n"
@@ -36,8 +38,8 @@ HELP_TEXT = (
 )
 
 
-LiszItem = Dict[str, Any]               #: node item data (nid) type
-LiszNode = List[LiszItem]               #: node/list type
+LiszItem = Dict[str, Any]           #: node item data (nid) type
+LiszNode = List[LiszItem]           #: node/list type
 
 
 def check_item_id(item_id: str) -> bool:
@@ -318,18 +320,36 @@ class LiszDataMixin(ABC):
             return searched_node[index]
         return dict(id='')
 
-    def move_item(self, dragged_node: LiszNode, item_id: str, node_idx: int):
-        """ move item from passed dragged_node list to index node_idx in :attr:`~LiszDataMixin.current_node_items`.
+    def move_item(self, dragged_node: LiszNode, dragged_id: str,
+                  dropped_path: Optional[List[str]] = None, dropped_id: str = '') -> bool:
+        """ move item id from passed dragged_node to the node and index specified by dropped_path and dropped_id.
 
-        :param dragged_node:    node where the item to moved from.
-        :param item_id:         id of the moved item.
-        :param node_idx:        index in the node where the item has to be moved to.
+        :param dragged_node:    node where the item got dragged/moved from.
+        :param dragged_id:      id of the dragged/moved item.
+        :param dropped_path:    optional destination/drop node path, if not passed use dragged_node.
+        :param dropped_id:      optional destination item where the dragged item will be moved before it.
+                                if empty string passed or not passed then the item will be placed at the end of
+                                the destination node.
         """
-        src_idx = self.find_item_index(item_id, searched_node=dragged_node)
-        nid = dragged_node.pop(src_idx)
-        if dragged_node == self.current_node_items and node_idx > src_idx:
-            node_idx -= 1
-        self.current_node_items.insert(node_idx, nid)
+        if dropped_path is None:
+            dropped_node = dragged_node
+        else:
+            dropped_node = self.flow_path_node(dropped_path)
+
+        src_node_idx = self.find_item_index(dragged_id, searched_node=dragged_node)
+        dst_node_idx = self.find_item_index(dropped_id, searched_node=dropped_node) if dropped_id else len(dropped_node)
+        assert src_node_idx >= 0 and dst_node_idx >= 0
+
+        if dragged_node != dropped_node and self.find_item_index(dragged_id, searched_node=dropped_node) != -1:
+            self.play_sound('error')
+            return False
+
+        nid = dragged_node.pop(src_node_idx)
+        if dragged_node == dropped_node and dst_node_idx > src_node_idx:
+            dst_node_idx -= 1
+        dropped_node.insert(dst_node_idx, nid)
+
+        return True
 
     def on_app_init(self):
         """ initialize self.current_node_items from flow_id/flow_path saved on last app stop. """
